from discord_self._vendor.discord.message import (
    Attachment,
    DeletedReferencedMessage,
    Message,
    MessageReference,
    PartialMessage,
    RoleSubscriptionInfo,
)

__all__ = [
    "Attachment",
    "DeletedReferencedMessage",
    "Message",
    "MessageReference",
    "PartialMessage",
    "RoleSubscriptionInfo",
]
