from discord_self._vendor.discord import discord
from discord_self._vendor.discord.__main__ import (
    add_newbot_args,
    add_newcog_args,
    core,
    main,
    newbot,
    newcog,
    parse_args,
    show_version,
    to_path,
)

__all__ = [
    "add_newbot_args",
    "add_newcog_args",
    "core",
    "discord",
    "main",
    "newbot",
    "newcog",
    "parse_args",
    "show_version",
    "to_path",
]
