from discord_self._vendor.discord.webhook.async_ import (
    PartialWebhookChannel,
    PartialWebhookGuild,
    Webhook,
    WebhookMessage,
)
from discord_self._vendor.discord.webhook.sync import SyncWebhook, SyncWebhookMessage

__all__ = [
    "PartialWebhookChannel",
    "PartialWebhookGuild",
    "SyncWebhook",
    "SyncWebhookMessage",
    "Webhook",
    "WebhookMessage",
]
