from discord_self._vendor.discord.utils import (
    MISSING,
    as_chunks,
    escape_markdown,
    escape_mentions,
    find,
    format_dt,
    get,
    maybe_coroutine,
    oauth_url,
    remove_markdown,
    set_target,
    setup_logging,
    sleep_until,
    snowflake_time,
    time_snowflake,
    utcnow,
)

__all__ = [
    "MISSING",
    "as_chunks",
    "escape_markdown",
    "escape_mentions",
    "find",
    "format_dt",
    "get",
    "maybe_coroutine",
    "oauth_url",
    "remove_markdown",
    "set_target",
    "setup_logging",
    "sleep_until",
    "snowflake_time",
    "time_snowflake",
    "utcnow",
]
