from discord_self._vendor.discord.types.integration import ConnectionIntegration
from discord_self._vendor.discord.types.snowflake import Snowflake
from discord_self._vendor.discord.types.user import (
    APIUser,
    Connection,
    ConnectionAccessToken,
    ConnectionAuthorization,
    ConnectionType,
    ConnectionVisibilty,
    GuildAffinities,
    GuildAffinity,
    Note,
    PartialConnection,
    PartialUser,
    PremiumType,
    ProtoSettings,
    ProtoSettingsType,
    Relationship,
    RelationshipType,
    User,
    UserAffinities,
    UserAffinity,
)

__all__ = [
    "APIUser",
    "Connection",
    "ConnectionAccessToken",
    "ConnectionAuthorization",
    "ConnectionIntegration",
    "ConnectionType",
    "ConnectionVisibilty",
    "GuildAffinities",
    "GuildAffinity",
    "Note",
    "PartialConnection",
    "PartialUser",
    "PremiumType",
    "ProtoSettings",
    "ProtoSettingsType",
    "Relationship",
    "RelationshipType",
    "Snowflake",
    "User",
    "UserAffinities",
    "UserAffinity",
]
